__all__ = ["NestedClassNameCharacterIterable"]

from typing import TypeAlias

from clsx.contrib.itertools.abc import NestedStrIterable


NestedClassNameCharacterIterable: TypeAlias = NestedStrIterable
